import axios from 'axios'
import Vue from 'vue'
import {
  getToken
} from '@/util/auth'
import {
  Toast
} from 'vant'

// 请求池-用于存储请求接口-防止重复提交
const pendingRequestPool = new Map()
// 间隔时间小于interval的,被视为重复提交的请求
const interval = 1000

console.log(process.env, process.env.VUE_APP_URL)
// 创建Axios实例
const request = axios.create({
  baseURL: process.env.VUE_APP_URL,
  timeout: 15000 // 60s
})

// Axios请求拦截器
request.interceptors.request.use(
  (config) => {
    // 在发送请求之前做些什么
    const isToken = Boolean((config.headers).isToken) // 根据请求配置是否需要为请求头添加token
    const isRepeated = Boolean((config.headers).isRepeated) // 根据请求配置判断是否需要防止重复提交请求
    Vue.$showLoading = true
    // Toast.loading({
    //  message: '加载中...',
    //  forbidClick: true
    // })
    let cancelFunction // 引用取消请求的cancelToken
    // 挂载请求key
    config.requestKey = `${config.method}-${config.url}`
    if (!!isToken && getToken()) {
      // 添加token到请求头
      config.headers['X-Token'] = getToken()
    }
    // console.log(config)
    if (!config.headers['Content-Type']) {
      config.headers['Content-Type'] = 'multipart/form-data'
    }
    if (isRepeated) {
      // 防止重复提交请求
      const requestObject = {
        url: config.url,
        data: typeof config.data === 'object'
          ? JSON.stringify(config.data)
          : config.data,
        time: Date.now()
      }
      const sessionObejct = pendingRequestPool.get(config.requestKey)

      config.cancelToken = new axios.CancelToken((cancel) => {
        cancelFunction = cancel
      })
      if (sessionObejct === undefined || sessionObejct === null) {
        pendingRequestPool.set(config.requestKey, requestObject)
      } else {
        // 请求池中存在相同的请求-根据时间间隔interval判断是否为重复的请求
        const {
          time
        } = sessionObejct
        if (Date.now() - time <= interval) {
          // 重复请求-取消当前请求
          cancelFunction()
          Promise.reject(new Error('数据正在处理，请勿重复提交'))
        }
      }
    }
    return Promise.resolve(config)
  },
  (error) => {
    // 对请求错误做些什么
    console.log(error)
    return Promise.reject(error)
  }
)

// Axios响应拦截器
request.interceptors.response.use(
  (response) => {
    const requestKey = response.config.requestKey
    pendingRequestPool.delete(requestKey)
    // 对不同的状态码进行判断处理
    const {
      code,
      data,
      msg
    } = response.data
    // 判断状态码
    if (code === -99) {
      window.sessionStorage.removeItem('MyUsersStore') // 移除sessionStorage中的缓存
      window.sessionStorage.clear()
      window.location.href = '/pwlogin'
      Toast.fail('登录失效，请重新登陆')
      return Promise.reject(new Error('会话过期，请重新登陆'))
    } else if (code === 500) {
      Toast.fail(msg)

      return Promise.reject(new Error(msg))
    } else if (code === 601) {
      Toast.fail(msg)

      return Promise.reject(new Error(msg))
    } else if (code !== 0) {
      Toast.fail(msg)

      return Promise.reject(new Error('未知错误,请联系管理员!'))
    } else {
      // Toast.loading({
      //  message: '加载中...',
      //  forbidClick: true
      // })
      return Promise.resolve(data)
    }
  },
  (error) => {
    // 超出 2xx 范围的状态码都会触发该函数。
    // 对响应错误做点什么
    Toast.fail(error.message)
    Vue.$showLoading = false
    return Promise.reject(error)
  }
)

// 导出Axios实例
export default request
